// -------------------------------------------------------------------------------
//
//     FxPro.com
//
// -------------------------------------------------------------------------------

using System;
using cAlgo.API;
using cAlgo.API.Indicators;
using System.Collections.Generic;

namespace cAlgo.Robots
{
    [Robot]
    public class BillWillamsTrader : Robot
    {
        [Parameter("CountBarsFractal",DefaultValue=10)]
        public int CountBarsFractal { get; set; } // количество баров из которых состоит фрактал
        
        [Parameter("MaxDistance",DefaultValue=10000)]
        public int MaxDistances { get; set; } // включение контроля расстояния от зеленой линии до точки входа (в пунктах)
        
        [Parameter("indent",DefaultValue=10)]
        public int indent { get; set; } // количество пунктов для отступа от максимума и минимума
        
        [Parameter("TypeEntry",DefaultValue=2)]
        public int Typeentry { get; set; } // тип входа после пробоя фрактала 1 - на текущем баре 2 - на баре закрытия 3 на откате к точке входа после пробоя
        
        [Parameter("RedContol",DefaultValue=1)]
        public int RedContols { get; set; } // контролировать находится ли пробойная цена выше ниже уровня красной линии
        
        [Parameter("jaw_period",DefaultValue=6)]
        public int jaw_period { get; set; } // Период усреднения синей линии (челюсти аллигатора)
        
        [Parameter("jaw_shift",DefaultValue=65)]
        public int jaw_shift { get; set; } // Смещение синей линии относительно графика цены
        
        [Parameter("teeth_period",DefaultValue=137)]
        public int teeth_period { get; set; } // Период усреднения красной линии (зубов аллигатора)
        
        [Parameter("teeth_shift",DefaultValue=136)]
        public int teeth_shift { get; set; } // Смещение красной линии относительно графика цены
        
        [Parameter("lips_period",DefaultValue=41)]
        public int lips_period { get; set; }
        
        [Parameter("lips_shift",DefaultValue=40)]
        public int lips_shift { get; set; } // Смещение зеленой линии относительно графика цены
        
        [Parameter("ma_method",DefaultValue=MovingAverageType.Simple)]
        public MovingAverageType ma_method { get; set; }
        
        [Parameter("TrendAligControl",DefaultValue=0)]
        public int TrendAligControls { get; set; } // включение контроля тренда по алигатору
        
        [Parameter("jaw_teeth_distense",DefaultValue=100)]
        public int jaw_teeth_distense { get; set; } // разница между зеленой и красной в пунктах
        
        [Parameter("teeth_lips_distense",DefaultValue=100)]
        public int teeth_lips_distense { get; set; } // разница между красной и синией в пунктах
        
        [Parameter("CloseDropTeeth",DefaultValue=2)]
        public int CloseDropTeeth { get; set; } // Включение закрытия сделки при косании или пробое челюсти. 0 - отключение 1 - по касанию 2 по закрытию бара
        
        [Parameter("CloseReversSignal",DefaultValue=2)]
        public int CloseReversSignal { get; set; } // Включение закрытия сделки при 1- образовании обратного фрактала 2 - при срабатывании обратного фрактала 0 выключено
        
        [Parameter("TrailingGragus",DefaultValue=1)]
        public int TrailingGragus { get; set; } // Включение трейлинг стопа по ценовому градусу наклона, если сильный угол то трейлинг по зеленой, если малый угол то трейлинг по красной
        
        [Parameter("smaperugol",DefaultValue=5)]
        public int smaperugol { get; set; }
        
        [Parameter("raznica",DefaultValue=5)]
        public int raznica { get; set; }
        
        [Parameter("StopLoss",DefaultValue=500)]
        public int StopLoss { get; set; }
        
        [Parameter("TakeProfit",DefaultValue=1650)]
        public int TakeProfit { get; set; }
        
        [Parameter("Volume",DefaultValue = 10000, MinValue = 0)]
        public int Volume { get; set; }
        
        private DataSeries median;
        private MovingAverage jawsMa;
        private MovingAverage teethMa;
        private MovingAverage lipsMa;
        private MovingAverage smaMa;
        private double oldopb,opb,ops,oldops, otkatb,otkats;
        private int fractalnew,vpravovlevo,numsredbar,colish;
        private List<Position> pos=new List<Position>();
        
        protected override void OnStart()
        {
            median=Indicators.MedianPrice().Result;
            jawsMa=Indicators.MovingAverage(median,jaw_period,ma_method);
            teethMa=Indicators.MovingAverage(median,teeth_period,ma_method);
            lipsMa=Indicators.MovingAverage(median,lips_period,ma_method);
            smaMa=Indicators.MovingAverage(MarketSeries.Close,smaperugol,MovingAverageType.Simple);
        }

        protected override void OnTick()
        {
            int last=MarketSeries.Close.Count-1;
            if(!(MarketSeries.Open[last]==MarketSeries.High[last] && MarketSeries.Open[last]==MarketSeries.Low[last])) return;
            ClassicFractal();
        }

        protected override void OnStop()
        {
            // Put your deinitialization logic here
        }
        protected override void OnPositionOpened(Position openedPosition)
        {
            double sl,tp;
            pos.Add(openedPosition);
            if(pos[pos.Count-1].TradeType== TradeType.Buy)
            {
            	sl=Symbol.Ask-StopLoss*Symbol.PointSize;
   				tp=Symbol.Ask+TakeProfit*Symbol.PointSize;
   				Trade.ModifyPosition(pos[pos.Count-1],sl,tp);
            }
            if(pos[pos.Count-1].TradeType== TradeType.Sell)
            {
            	sl=Symbol.Bid+StopLoss*Symbol.PointSize;
   				tp=Symbol.Bid-TakeProfit*Symbol.PointSize;
   				Trade.ModifyPosition(pos[pos.Count-1],sl,tp);
            }
        }
        
        protected override void OnPositionClosed(Position position)
        {
        	for(int i=0;i<pos.Count;i++)
        		if(pos[i].Id==position.Id)
        		{
        			pos.RemoveAt(i);
        			break;
        		}
        }
        
        private void ClassicFractal()
        {
        	int buy,sell;
        	
        	
        	ClassicFractalPosManager();
        	
        	buy=0;sell=0;
        	for(int  i=0;i<pos.Count;i++)
        	{
        		if(pos[i].TradeType==TradeType.Buy){buy=1;}
      			if(pos[i].TradeType== TradeType.Sell){sell=1;}
      		}
      		//найдем скоьлко смотреть вправо и в лево
      		int last=MarketSeries.Close.Count-1;
   			vpravovlevo=(CountBarsFractal-1)/2;
   			numsredbar=(CountBarsFractal-vpravovlevo);
   			colish=numsredbar-1;
   			/*----------------------------------------ПОКУПКА------------------------------------------*/
   
   			//найдем фрактал на покупку
   			if(MarketSeries.High[numsredbar]>MarketSeries.High[iHighest(colish,last-numsredbar+1)] && MarketSeries.High[last-numsredbar]>MarketSeries.High[iHighest(colish,last-1)] && (RedContol(MarketSeries.Close[last-1],0) && RedContols==1))
   			{
   				opb=MarketSeries.High[numsredbar]+indent*Symbol.PointSize;
   			}
    		//проверка входа на касании или по закрытию бара
   			if(buy==0 &&  ((Symbol.Ask>opb && Typeentry==1 ) || (MarketSeries.Close[last-1]>opb && Typeentry==2)) 
   			&& opb!=oldopb && MaxDistance(opb) && opb>0 
   			&& ((RedContol(MarketSeries.Close[last-1],0) && RedContols==1) || RedContols==0)
   			&& ((TrendAligControl(0) && TrendAligControls==1) || TrendAligControls==0))
   			{
   			oldopb=opb;

   			Trade.CreateMarketOrder(TradeType.Buy,Symbol,Volume);
   			
   			}
   			 /*------------------------------------------ПРОДАЖА----------------------------------------*/
   
   			//найдем фрактал на продажу
   			if(MarketSeries.Low[numsredbar]<MarketSeries.Low[iLowest(colish,last-numsredbar+1)] && MarketSeries.Low[last-numsredbar]<MarketSeries.Low[iLowest(colish,last)]  && (RedContol(MarketSeries.Close[last-1],1) && RedContols==1) )
   			{
   				ops=MarketSeries.Low[numsredbar]-indent*Symbol.PointSize;
   			}
   			//проверка входа на касании или по закрытию бара
   			if(sell==0 && ( (Symbol.Bid<ops && Typeentry==1) ||  (MarketSeries.Close[last-1]<ops && Typeentry==2))   
   			&& oldops!=ops && MaxDistance(ops) 
   			&& ((RedContol(MarketSeries.Close[last-1],1) && RedContols==1) ||RedContols==0)
   			&& ((TrendAligControl(1) && TrendAligControls==1) || TrendAligControls==0))
   			{
   				oldops=ops;
   				
   				Trade.CreateMarketOrder(TradeType.Sell,Symbol,Volume);
   				
   			}
   		}
   		private int iHighest(int cnt, int start)
		{
			int k=start;
			double hg=MarketSeries.High[start];
			for(int i=0; i<cnt; i++)
			{
				if(hg<MarketSeries.High[start-i]) { hg=MarketSeries.High[start-i]; k=start-i; }
			}
			return k;
		}
		private int iLowest(int cnt, int start)
		{
			int k=start;
			double lw=MarketSeries.Low[start];
			for(int i=0;i<cnt;i++)
			{
				if(lw>MarketSeries.Low[start-i]) { lw=MarketSeries.Low[start-i]; k=start-i; }
			}
			return k;
		}
		
		private bool MaxDistance(double entryprice)
		{
 
			double lips=lipsMa.Result[lipsMa.Result.Count-2-lips_shift];
 
			if(Math.Abs(entryprice-lips)<MaxDistances*Symbol.PointSize){return true;}
			return false;
		}
		private bool RedContol(double entryprice,int  type)
		{
 
			double teeth=teethMa.Result[teethMa.Result.Count-2-teeth_shift];
 
			if(entryprice>teeth && type==0){return(true);}
			if(entryprice<teeth && type==1){return(true);}
			return(false);
		}
		private bool TrendAligControl(int type)
		{
 
			double teeth=teethMa.Result[teethMa.Result.Count-2-teeth_shift];
			double lips=lipsMa.Result[lipsMa.Result.Count-2-lips_shift];
			double jaw=jawsMa.Result[jawsMa.Result.Count-2-jaw_shift];
 
 
			if(type==0 && lips-teeth>teeth_lips_distense*Symbol.PointSize && teeth-jaw>jaw_teeth_distense*Symbol.PointSize){return true;}
			if(type==1 && teeth-lips>teeth_lips_distense*Symbol.PointSize && jaw-teeth>jaw_teeth_distense*Symbol.PointSize){return true;}
 
 
			return false;
		}
		private void ClassicFractalPosManager()
		{
			int last=MarketSeries.Close.Count-1;
			int i,buy,sell;
			double jaw=jawsMa.Result[jawsMa.Result.Count-2-jaw_shift];
			double teeth=teethMa.Result[teethMa.Result.Count-2-teeth_shift];
			double lips=lipsMa.Result[lipsMa.Result.Count-2-lips_shift];
			double lipsl=lipsMa.Result[lipsMa.Result.Count-3-lips_shift];
			double sma=smaMa.Result[smaMa.Result.Count-2];
			double smal=smaMa.Result[smaMa.Result.Count-3];
 
      		buy=0;
      		sell=0;
      		for(int  i2=0;i2<pos.Count;i2++)
        	{
        		if(pos[i2].TradeType==TradeType.Buy){buy=1;}
      			if(pos[i2].TradeType== TradeType.Sell){sell=1;}
      		}
 
   			for( i=0; i<pos.Count; i++)          
     		{
         		if(pos[i].TradeType==TradeType.Buy && ((CloseDropTeeth==1 && Symbol.Bid<=jaw ) ||  (CloseDropTeeth==2 && MarketSeries.Close[last-1]<=jaw )))
            		{ 
            			Trade.Close(pos[i]);
            			
            			return;
            		}
          	
          
         	  	if(pos[i].TradeType==TradeType.Buy && 
            	((CloseReversSignal==1 && MarketSeries.Low[numsredbar]<MarketSeries.Low[iLowest(colish,last-numsredbar+1)] && MarketSeries.Low[last-numsredbar]<MarketSeries.Low[iLowest(colish,last)] ) 
            	||(CloseReversSignal==2 && sell==1 )))
            	{ 
            		Trade.Close(pos[i]);
					            		
            		return;
            	}
          	
          
        	
            	if(pos[i].TradeType==TradeType.Buy && TrailingGragus==1 && lips-lipsl>sma-smal && pos[i].Profit>0)
             	{
             		if(pos[i].StopLoss<lips)
              		{
              			Trade.ModifyPosition(pos[i],lips,pos[i].TakeProfit);
              			
              			return;
              		}  
             	} 
         	
           
           
        	
            	if(pos[i].TradeType==TradeType.Buy && TrailingGragus==1 && lips-lipsl<=sma-smal && pos[i].Profit>0)
             	{
             		if(pos[i].StopLoss<teeth || lips>teeth)
              		{
              			Trade.ModifyPosition(pos[i],teeth,pos[i].TakeProfit);
              			return;
              		}  
             	} 
          	
         
         	
           		if(pos[i].TradeType==TradeType.Sell && ((CloseDropTeeth==1 && Symbol.Ask>=jaw ) ||  (CloseDropTeeth==2 && MarketSeries.Close[last-1]>=jaw )))
           		{
           			Trade.Close(pos[i]);
					           			
           			return;
           		}
         	
          
          	 
           		if(pos[i].TradeType==TradeType.Sell && ((CloseReversSignal==1 && MarketSeries.High[last-numsredbar]>MarketSeries.High[iHighest(colish,last-numsredbar+1)] && MarketSeries.High[last-numsredbar]>MarketSeries.High[iHighest(colish,last-1)]) 
           		||  (CloseReversSignal==2 && buy==1  )))
          		{
           			Trade.Close(pos[i]);
           			
           			return;
           		}
         	    
         
         
        	
            	if(pos[i].TradeType==TradeType.Sell && TrailingGragus==1 && lipsl-lips<smal-sma && pos[i].Profit>0)
             	{
             		if(pos[i].StopLoss>lips)
              		{
              			Trade.ModifyPosition(pos[i],lips,pos[i].TakeProfit);
              			return;
              		}  
             	} 
          	
           
           
        	
            	if(pos[i].TradeType==TradeType.Sell && TrailingGragus==1 && lipsl-lips>smal-sma && pos[i].Profit>0)
             	{
             		if(pos[i].StopLoss>teeth || lips<teeth)
              		{
              			Trade.ModifyPosition(pos[i],teeth,pos[i].TakeProfit);
              			return;
              		}  
             	} 
          	
                      
       	}
       
 
		}
    }
}